/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.autoloader;

import java.io.File;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.microkernel.api.communication.JMXService;
import org.ow2.petals.microkernel.api.jbi.management.DeploymentService;
import org.ow2.petals.microkernel.api.jbi.management.InstallationService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * Periodically check if new Install or Deploy elements are presents in the
 * autoloader directories.
 * 
 * @author Adrien Louis - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = DirectoryScanner.class))
public class InstallDirectoryScannerImpl implements DirectoryScanner {

    @Requires(name = "deployment")
    private DeploymentService deploymentService;

    @Requires(name = "installation")
    private InstallationService installationService;

    @Requires(name = "jmx")
    private JMXService jmxService;

    private LoggingUtil logger;

    /**
     * The task scanning the directory
     */
    private TimerTask scanningTask;

    public InstallDirectoryScannerImpl() {
        // NOP
    }

    /**
     * Starts the installation directory scanner.
     */
    @LifeCycle(on = LifeCycleType.START)
    private void start() {
        this.logger = new LoggingUtil(
                Logger.getLogger(AutoLoaderService.FRACTAL_COMPONENT_LOGGER_NAME + "."
                        + InstallDirectoryScannerImpl.class.getSimpleName()));
        this.logger.call();
    }

    public void startScanner(final File installDirectory, final Timer directoriesToScanTimer,
            final long scanPeriod) {

        this.logger.start();

        this.scanningTask = new InstallDirectoryScanningTask(this.deploymentService,
                this.installationService, this.jmxService, installDirectory, this.logger);

        directoriesToScanTimer.schedule(this.scanningTask, scanPeriod, scanPeriod);

        this.logger.end();
    }

    /**
     * Stop the autoload of the components. TODO : synchronization, does the
     * autoload must wait before the end of an installation before closing?
     */
    @LifeCycle(on = LifeCycleType.STOP)
    private void stop() {

        this.logger.start();

        this.scanningTask.cancel();

        this.logger.end();
    }

}
